# -*- coding: utf-8 -*-
"""Script for printing trivial statistics about inbox, such as:
	Number of texts
	Number of unique contacts who sent the texts
	Number of texts sent by respective contacts
"""

__author__ = "Kamran Riaz Khan"
__email__ = "krkhan@inspirated.com"
__version__ = "1.0"
__copyright__ = "Copyright (c) 2009 Kamran Riaz Khan"
__license__ = "Python"
__status__ = "Production"

import appuifw, e32, graphics, inbox

def exit_key_handler():
	"Release app_lock."
	app_lock.signal()

def parse_inbox_stats(stats):
	"""Parse the inbox statistics,
	Updates the stats dictionary with:
		sms-count : Number of texts
		sms-contacts: List of tuples with following pairs:
			Name of contact, Number of corresponding
			(ordered according to decreasing number of texts)"""
	curr_inbox = inbox.Inbox()
	messages = curr_inbox.sms_messages()
	contacts = {}

	for i in messages:
		address = curr_inbox.address(i)
		contacts[address] = contacts.get(address, 0) + 1
	
	contacts = contacts.items()
	contacts.sort(lambda x, y: cmp(x[1], y[1]))
	contacts.reverse()

	stats["sms-count"] = len(messages)
	stats["sms-contacts"] = contacts

def print_inbox_stats(content, stats):
	"""Print inbox stats in the content Text field,
	Remembers the cursor position of Text before the call
	and points at it again after updating the content."""
	pos = content.get_pos()

	statsmap = [
		(u"SMS Count", unicode(stats["sms-count"])),
		(u"Unique Contacts", unicode(len(stats["sms-contacts"]))),
		(u"", u"")
		]

	statsmap += [(k, unicode(v)) for k, v in stats["sms-contacts"]]

	for i in statsmap:
		content.style = appuifw.STYLE_BOLD
		content.add(i[0] + (i[0] and u": " or u""))
		content.style = 0
		content.add(i[1] + u"\n")

	content.set_pos(pos)

def draw_inbox_stats(canvas, stats):
	"""Draw inbox stats.
	
	Keyword arguments:
		canvas -- instance of ScrolledCanvas
		stats -- inbox stats parsed by parse_inbox_stats()"""
	import roundedrectangle

	def resize_image(image, width, height):
		"Return resized image of width x height"
		newimage = graphics.Image.new((width, height))
		newimage.blit(image)
		return newimage

	canvas._image
	origwidth, origheight = canvas._image.size

	highest = float(stats["sms-contacts"][0][1])
	statsmap = [
		(u"SMS Count", unicode(stats["sms-count"]), 1),
		(u"Unique Contacts",
			unicode(len(stats["sms-contacts"])), 1),
		(u"", u"", 0)
		]
	statsmap += [(k, unicode(v), v / highest)
		for k, v in stats["sms-contacts"]]

	i = 1
	height = origheight
	margin = 10
	ystep = 30
	gradstep = 200 / len(statsmap)
	kcenter = (origwidth - 2 * margin - 40) / 2
	# key, value, weight
	for k, v, w in statsmap:
		if ystep * i >= height:
			height += origheight
			canvas._image = resize_image(canvas._image,
				origwidth, height)

		if k == u"":
			i = i + 1
			continue

		karea = ((margin,
			ystep * (i - 1) + margin),
			(kcenter + kcenter * w,
			ystep * i))
		varea = ((origwidth - margin - 40,
			ystep * (i - 1) + margin),
			(origwidth - margin,
			ystep * i))

		roundedrectangle.rounded_rectangle_text(canvas._image,
			karea, text = k, radius = 2,
			fill = (255, i * gradstep, i * gradstep))
		roundedrectangle.rounded_rectangle_text(canvas._image,
			varea, text = v, radius = 2,
			fill = (255, i * gradstep, i * gradstep))

		i = i + 1

	canvas._image = resize_image(canvas._image, origwidth, ystep * i)
	canvas.redraw_canvas((0, 0, origwidth, origheight))

def switch_to_graphics(stats):
	image = graphics.Image.new(appuifw.app.layout(appuifw.EMainPane)[0])
	canvas = scrolledcanvas.ScrolledCanvas(image)
	draw_inbox_stats(canvas, stats)
	appuifw.app.body = canvas
	appuifw.app.menu = [(u"Switch to text",
		lambda: switch_to_text(stats))]

def switch_to_text(stats):
	content = appuifw.Text()
	print_inbox_stats(content, stats)
	appuifw.app.body = content
	appuifw.app.menu = [(u"Switch to graphics",
		lambda: switch_to_graphics(stats))]

if __name__ == "__main__":
	import thread
	import scrolledcanvas

	content = appuifw.Text()

	appuifw.app.title = u'Inbox Stats'
	appuifw.app.body = content

	stats = {}
	t = thread.start_new_thread(parse_inbox_stats, (stats,))

	content.style = appuifw.STYLE_ITALIC
	content.add(u"Processing text messages...\n")
	thread.ao_waittid(t)
	content.add(u"Done!\n\n")
	content.style = 0

	print_inbox_stats(content, stats)
	appuifw.app.menu = [(u"Switch to graphics",
		lambda: switch_to_graphics(stats))]
	appuifw.app.exit_key_handler = exit_key_handler

	app_lock = e32.Ao_lock()
	app_lock.wait()

