# -*- coding: utf-8 -*-
"A Canvas class with built-in support for scrolling"

__author__ = "Kamran Riaz Khan"
__email__ = "krkhan@inspirated.com"
__version__ = "1.0"
__copyright__ = "Copyright (c) 2009 Kamran Riaz Khan"
__license__ = "Python"
__status__ = "Production"

import appuifw, key_codes

class ScrolledCanvas(appuifw.Canvas):
	"""Inherited Canvas with support for scrolling."""
	MOVE = {
		'up': {'x': 0, 'y': -10},
		'down': {'x': 0, 'y': 10},
		'left': {'x': -10, 'y': 0},
		'right': {'x': 10, 'y': 0}
	}

	def __init__(self, image):
		"""Creates new ScrolledCanvas.
		
		Keyword arguments:
			image -- surface to be scrolled"""
		appuifw.Canvas.__init__(self,
			redraw_callback = self.redraw_canvas)

		self._image = image
		self._source_area = ((0, 0), self.size)
		self.bind(key_codes.EKeyUpArrow,
			lambda: self.scroll_canvas('up'))
		self.bind(key_codes.EKeyDownArrow,
			lambda: self.scroll_canvas('down'))
		self.bind(key_codes.EKeyLeftArrow,
			lambda: self.scroll_canvas('left'))
		self.bind(key_codes.EKeyRightArrow,
			lambda: self.scroll_canvas('right'))

	def redraw_canvas(self, redraw_area):
		"Callback method for redrawing canvas."
		self.blit(self._image, source = self._source_area)

	def scroll_canvas(self, direction):
		"""Callback method for scrolling canvas.

		Keyword arguments:
			direction -- 'up', 'down', 'left' or 'right'"""
		image_area = ((0, 0), self._image.size)
		new_area = [(x + self.MOVE[direction]['x'],
			y + self.MOVE[direction]['y'])
			for x, y in self._source_area]

		if new_area[0][0] >= image_area[0][0] \
			and new_area[0][1] >= image_area[0][1] \
			and new_area[1][0] <= image_area[1][0] \
			and new_area[1][1] <= image_area[1][1]:
			self._source_area = new_area

		self.redraw_canvas(((0, 0), self.size))

def exit_key_handler():
	"Release app_lock."
	app_lock.signal()

if __name__ == "__main__":
	import e32, graphics

	image = graphics.Image.new((320, 240))
	canvas = ScrolledCanvas(image)

	appuifw.app.title = u'Scrolled Canvas'
	appuifw.app.body = canvas
	appuifw.app.exit_key_handler = exit_key_handler

	image.text((10, 20), u"This is a scrolled canvas")
	image.text((10, 40), u"Use arrow keys to scroll")

	app_lock = e32.Ao_lock()
	app_lock.wait()

